<?php 
class WCAM_Notifier
{
	var $current_notification_name;
	var $hooks;
	var $attachments_to_email_already_performed = false;
	var $text_embed_to_email_already_performed = false;
	public function __construct()
	{
		
		add_action( 'woocommerce_email_before_order_table', array( &$this, 'add_wcam_data' ), 10,4 );
		add_filter( 'woocommerce_email_attachments', array( &$this, 'add_wcam_attachment' ), 10, 3);
	}
	private function get_order_options($order)
	{
		$wcam_options = get_post_meta( WCAM_Order::get_id($order), '_wcam_options');
		$wcam_options = isset($wcam_options) && isset($wcam_options[0]) ? $wcam_options[0]:null;
		return $wcam_options;
	}
	
	private function get_order_products_attachments_metadata($order)
	{
		global $wcam_product_model;
		return $wcam_product_model->get_attachments_per_order_id(WCAM_Order::get_id($order));
	}
	public function get_order_statuses()
	{
		
		$result = array();
		if(function_exists( 'wc_get_order_statuses' ))
		{
			
			$result['version'] = 2.2;
			$result['statuses'] = array();
			$temp  = wc_get_order_statuses();
			foreach($temp as $slug => $title)
			{
					$result['statuses'][] = array('status' => 'wc-' === substr( $slug, 0, 3 ) ? substr( $slug, 3 ) : $slug, 'id' =>  $slug);
			}
		}
		else
		{
			$args = array(
				'hide_empty'   => false, 
				'fields'            => 'id=>slug', 
			);
			$result['version'] = 2.1;
			$result['statuses'] = array();
			$temp = get_terms('shop_order_status', $args);
			foreach($temp as $id => $slug)
			{
					$result['statuses'][] = array('status' => $slug, 'id' =>  $id);
			}
		}
		return $result;
	}
	public function get_order_statuses_id_to_name()
	{
		$result = array();
		if(function_exists( 'wc_get_order_statuses' ))
		{
			
			$result['version'] = 2.2;
			$result['statuses'] = wc_get_order_statuses();
		}
		else
		{
			$args = array(
				'hide_empty'   => false, 
				'fields'            => 'id=>name', 
			);
			$result['version'] = 2.1;
			$result['statuses'] =  get_terms('shop_order_status', $args);
		}
		return $result;
	}
	public function set_current_notification_name($order_id)
	{
		foreach ( $this->hooks as $hooks ) 
		{
            if( did_action( $hooks['hook'] ) > 0 ) 
			{
 
              $this->current_notification_name = $hooks['id'] ;
            } 
			
        } 
	}
	static function send_user_email($meta_info, $file_order_metadata, $order_id)
	{
		global $wcam_order_model;
		if(isset($meta_info['wcam-notification-message']) && isset($meta_info['wcam-notification-subject']) && isset($meta_info['wcam-notification-recipient']))
			{
				$item_to_attach = isset($meta_info['wcam-attachment-ids']) ? explode(",",$meta_info['wcam-attachment-ids']):null;
				$email_sender = new WCAM_Email();
				$order = wc_get_order($order_id);
				$user_email = $meta_info['wcam-notification-recipient']; //To force using user billing email: WCAM_Order::get_billing_email($order);
				$message =  $meta_info['wcam-notification-message'] ;
				$attachments = array();
				
				
				//Download links 
				foreach($file_order_metadata as $id => $meta)
				{
					$id = (int)$id;
					$replaced_shortcode = '[wcam_download_link attachment_id="'.$id.'" order_id="'.WCAM_Order::get_id($order).'"]';
					$message = str_replace('[wcam_download_link id="'.$id.'"]', $replaced_shortcode, stripslashes (htmlspecialchars_decode($message)));
				}
				
				//order link embed
				$myaccount_page_id = get_option( 'woocommerce_myaccount_page_id' );
				
				$subject = str_replace('[order_id]', apply_filters('wcam_get_visual_order_id', WCAM_Order::get_id($order)), $meta_info['wcam-notification-subject']);
				$message = str_replace('[wcam_order_page_link]', '[wcam_order_page_link page_id="'.$myaccount_page_id.'" order_id="'.WCAM_Order::get_id($order).'"]', stripslashes (htmlspecialchars_decode($message)));
				
				//attachment
				if(isset($item_to_attach))
					foreach($item_to_attach as $id)
					{
						if($wcam_order_model->is_attachment_expired($file_order_metadata[$id], WCAM_Order::get_date_created($order)))
							continue;
						
						if(isset($file_order_metadata[$id]['absolute_path']) && file_exists($file_order_metadata[$id]['absolute_path'])) //uploaded file
							array_push($attachments, $file_order_metadata[$id]['absolute_path']); 
						else if(isset($file_order_metadata[$id]['media_from_gallery']) && $file_order_metadata[$id]['media_from_gallery'] == 'yes' && isset($file_order_metadata[$id]['media_id']))
						{
							array_push($attachments, get_attached_file( $file_order_metadata[$id]['media_id'] ));
						}
						
					}
					
				$message = $wcam_order_model->replace_shortcodes_with_order_data($message, $order );
				
				$email_sender->trigger($user_email, $subject,  do_shortcode($message), $attachments);
				$attachment_list_text = "";
				if(count($attachments))
				{
					$attachment_list_text = esc_html__("The following file were attached: ", 'woocommerce-attach-me');
					$attachment_list_text .= "<ol>";
					foreach($attachments as $attachment_name)
						$attachment_list_text .= "<li>".basename($attachment_name)."</li>";
					$attachment_list_text .= "</ol>";
				}
				$order->add_order_note(wcam_html_escape_allowing_special_tags(sprintf(__("Attach Me! - Manual notification was successfully sent. %s", 'woocommerce-attach-me'), $attachment_list_text), false));
				
				$results = esc_html__('Message has been sent successfully', 'woocommerce-attach-me');
			}
			else 
				$errors = esc_html__('Subject and message cannot be empty.', 'woocommerce-attach-me');
			
		return isset($results) ? $results : $errors;
	}
	public function add_wcam_data($order, $sent_to_admin = false, $plain_text = false, $email = null )
	{
		if(!isset($order))
			return;
		
		global $wcam_order_model, $wcam_shortcode_model;
		try
		{
			$wcam_options = $this->get_order_options($order);
		}
		catch(Exception $e)
		{
			wcam_write_log('Error on while attaching files via the "woocommerce_email_before_order_table" action. The WCAM_Order component has not been initialized because one of the other plugins is interfering. Try disabling all of them and re-enable one by one until you find the interfering one.');
			return;
		}
		$wcam_meta_data = $wcam_order_model->get_attachments_metadata(WCAM_Order::get_id($order));
		$current_status = "wc-".$order->get_status( );
		$order_id = WCAM_Order::get_id($order);
		
		if(isset($wcam_options) && $wcam_options['embed-links-to-complete-mail'] == "yes" && 
		   !isset($wcam_options['embed-links-hide-by-status']) || empty($wcam_options['embed-links-hide-by-status']) || !in_array($current_status, $wcam_options['embed-links-hide-by-status']))
		{
			$message =  wcam_get_value_if_set($wcam_options,'embed-links-to-complete-mail-text', "") ; 
			//Download links 
			if(isset($wcam_meta_data))
				foreach($wcam_meta_data as $id => $meta)
				{
					$id = (int)$id;
					$replaced_shortcode =  '[wcam_download_link attachment_id="'.$id.'" order_id="'.WCAM_Order::get_id($order).'"]';
					$message = str_replace('[wcam_download_link id="'.$id.'"]', $replaced_shortcode,stripslashes ( htmlspecialchars_decode($message)));
				}
				
			//order link embed
			$myaccount_page_id = get_option( 'woocommerce_myaccount_page_id' );
			$message = str_replace('[wcam_order_page_link]', '[wcam_order_page_link page_id="'.$myaccount_page_id.'" order_id="'.WCAM_Order::get_id($order).'"]', stripslashes (htmlspecialchars_decode($message)));
			$message = $wcam_order_model->replace_shortcodes_with_order_data($message, $order);
			
			if(!empty($message))
			{
				echo "<div style='display:block; margin-top:10px; margin-bottom:10px; border:1px #dfdfdf solid; padding:10px 10px 10px 20px;'>";
				echo do_shortcode($message);
				echo "</div>";
			}
			
			
			//Product attachments 
			$all_product_attachments = $this->get_order_products_attachments_metadata($order);
			$product_attachment_email_data = array();
			if(!empty($all_product_attachments ) && (!isset( $status ) || $status != 'new_order' ))
			{
				foreach($all_product_attachments as $product_attachments)
				{
					foreach($product_attachments as $product_attachment)
						 if(!$product_attachment['has_expired'] && $product_attachment['attachment_visibility'] != 'product_page' && $product_attachment['email_description'] != '')
						 {
							$product_attachment_email_data[$product_attachment['id']] = $product_attachment;
						 }
				}
				if(!empty($product_attachment_email_data))
					include WCAM_PLUGIN_ABS_PATH.'/templates/email_product_attachment_description.php';
			}
		
			//Order note
			if(!$this->text_embed_to_email_already_performed && !empty($message))
			{
				$this->text_embed_to_email_already_performed = true;
				$order->add_order_note(wcam_html_escape_allowing_special_tags(sprintf(__('Attach Me! - Attachment text was successfully embedded to <strong>%s order</strong> notification email.', 'woocommerce-attach-me'),$order->get_status( )), false));			
			}
		}
	}
	public function add_wcam_attachment( $attachments , $status, $order ) 
	{
		global $wcam_order_model;
		
		
		$reflect = is_object($order) ? new ReflectionClass($order) : "none";
		if(!isset($order) || !is_object ($order) || (get_class($order) != 'WC_Order' && get_class($order) != 'WC_Admin_Order' && $reflect->getShortName() != 'Order'))
			return $attachments;
		
		if(!isset($wcam_order_model))
		{
			wcam_write_log('Error on while attaching files via the "woocommerce_email_attachments" action. The $wcam_order_model component has not been initialized because one of the other plugins is interfering. Try disabling all of them and re-enable one by one until you find the interfering one.');
			return;
		}
		/* 
			new_order
			customer_on_hold_order
			customer_processing_order
			customer_completed_order
			customer_invoice
			customer_note
			low_stock
			no_stock
			backorder
			customer_new_account
			customer_invoice_paid
			*/
		
		//Order attachments
		$attachment_names = array();
		$any_attachment_was_included = false;
		
		if( isset( $status ) ) 
		{
			$wcam_meta_data = $wcam_order_model->get_attachments_metadata(WCAM_Order::get_id($order));
			//attachment
			
			
			if(isset($wcam_meta_data))
				foreach($wcam_meta_data as $id => $meta)
				{
					if( ( $status == 'customer_processing_order' && isset($meta['attach-file-to-processing-order-email']) && $meta['attach-file-to-processing-order-email'] == "yes") ||
					   ($status == 'customer_completed_order' && isset($meta['attach-file-to-complete-order-email']) && $meta['attach-file-to-complete-order-email'] == "yes") ||
					   ($status == 'customer_invoice' && isset($meta['attach-file-to-customer-invoice-email']) && $meta['attach-file-to-customer-invoice-email'] == "yes") ||
					   ($status == 'customer_refunded_order' && isset($meta['attach-file-to-order-refunded-email']) && $meta['attach-file-to-order-refunded-email'] == "yes") ||
					   (isset($meta['wc_custom_order_emails_attachment'][$status]) && $meta['wc_custom_order_emails_attachment'][$status] == 'yes'))
					{
						
						if($wcam_order_model->is_attachment_expired($meta, WCAM_Order::get_date_created($order)))
							continue;
						
						$any_attachment_was_included = true;
						$file_name = isset($meta['absolute_path']) && $meta['absolute_path'] != "" ? $meta['absolute_path'] : $meta['url'];
						$attachment_names[] = esc_html__("[Order attachment]", 'woocommerce-attach-me')." ".$meta['title'].": ".basename($file_name);
						
						if(isset($meta['absolute_path']) && file_exists($meta['absolute_path'])) //uploaded file
							array_push($attachments, $meta['absolute_path']); 
						else if(isset($meta['media_from_gallery']) && $meta['media_from_gallery'] == 'yes' && isset($meta['media_id']))
						{
							array_push($attachments, get_attached_file( $meta['media_id'] ));
						}
					}
					
				}	
		}
		
		//Automatic products attachments
		$all_attachments = $this->get_order_products_attachments_metadata($order);
		if(!empty($all_attachments ) && (!isset( $status ) || $status != 'new_order' ))
		{
			foreach($all_attachments as $product_attachments)
			{
				foreach($product_attachments as $product_attachment)
					 if(!$product_attachment['has_expired'] && $product_attachment['attachment_visibility'] != 'product_page')
					 {
						$any_attachment_was_included = true;
						$file_data = pathinfo($product_attachment['file_path']); //Get file data (like file name)
						$attachment_names[$product_attachment['id']] = esc_html__("[Product attachment]", 'woocommerce-attach-me')." ".$product_attachment['file_name'];
						$attachments[$file_data['filename']] = $product_attachment['file_path'];
					 }
			}
		}
		
		//Automatic attachment
		$option = new WCAM_Option();
		$automatic_attachments = $option->get_automatic_attacments_options();
		if(!empty($automatic_attachments))
			foreach($automatic_attachments["automatic_email_attachments"] as $automatic_attachment)
			{
				
				if( isset( $status ) && in_array ( $status, $automatic_attachment['email_type'] ) ) 
				{
					$any_attachment_was_included = true;
					$attachment_names[$automatic_attachment['selected_file']] = esc_html__("[Automatic email attachment]", 'woocommerce-attach-me')." ".@basename(get_attached_file( $automatic_attachment['selected_file'] ));
					array_push($attachments, get_attached_file( $automatic_attachment['selected_file'] ) );
				}
			}
		$this->attachments_to_email_already_performed = true;
		
		
		//Order note
		if($any_attachment_was_included)
		{
			$attachment_list_text = "<ol>";
			foreach($attachment_names as $attachment_name)
				$attachment_list_text .= "<li>{$attachment_name}</li>";
			$attachment_list_text .= "</ol>";
			
			$order->add_order_note(wcam_html_escape_allowing_special_tags(sprintf(__("Attach Me! - The following attachments were successfully included into %s order notification email: {$attachment_list_text}", 'woocommerce-attach-me'),$order->get_status( )), false));
		}
		return $attachments;
	}
	private function var_debug($var)
	{
		echo "<pre>";
		var_dump($var);
		echo "</pre>";
	}
}
?>